"""Exceptions module for the PyVSS."""

from typing import Dict, Optional


class VssError(RuntimeError):
    """Vss Error module."""

    _http_code: int = 0
    message: str
    http_headers: Dict = {}

    @property
    def http_code(self):
        """Get HTTP Code."""
        return self._http_code

    @http_code.setter
    def http_code(self, val):
        """Set HTTP Code."""
        self._http_code = int(val)

    @http_code.deleter
    def http_code(self):
        """Delete HTTP Code."""
        del self._http_code

    def __init__(
        self,
        message: str,
        http_code: Optional[int] = 0,
        http_headers: Optional[Dict] = None,
    ):
        """Initialize class."""
        self.message = message
        self.http_code = http_code
        if http_headers is not None:
            self.http_headers = http_headers
        super().__init__(self.message)

    def __str__(self):
        """Get string representation."""
        return f'{self.message} HTTP Code: {self.http_code}'



import codecs
from datetime import datetime as dt
import io
import os
import re

from setuptools import find_packages, setup

# shared consts using approach suggested at
# https://stackoverflow.com/questions/17583443/what-is-the-correct-way-to-share-package-version-with-setup-py-and-the-package


def read(*parts):
    """Read file from current directory."""
    here = os.path.abspath(os.path.dirname(__file__))
    with codecs.open(os.path.join(here, *parts), 'r') as infofile:
        return infofile.read()


def find_version(*file_paths):
    """Locate version info to share between const.py and setup.py."""
    version_file = read(*file_paths)  # type: ignore
    version_match = re.search(
        r"^__version__ = ['\"]([^'\"]*)['\"]", version_file, re.M
    )
    if version_match:
        return version_match.group(1)
    raise RuntimeError("Unable to find version string.")


def load_requirements(requires_file='requirements.txt'):
    """Load requirements from file"""
    with open(requires_file, encoding='utf-8') as f:
        return f.read().splitlines()


__VERSION__ = find_version("pyvss", "const.py")  # type: ignore

REQUIRED_PYTHON_VER = (3, 7, 5)

REQUIRES = load_requirements()

PACKAGES = find_packages(exclude=['tests', 'tests.*'])

PROJECT_NAME = 'ITS Private Cloud Python Client'
PROJECT_PACKAGE_NAME = 'pyvss'
PROJECT_LICENSE = 'MIT'
PROJECT_AUTHOR = 'University of Toronto'
PROJECT_COPYRIGHT = f' 2019-{dt.now().year}, {PROJECT_AUTHOR}'
PROJECT_URL = 'https://gitlab-ee.eis.utoronto.ca/vss/py-vss'
PROJECT_DOCS = 'https://eis.utorotno.ca/~vss/py-vss'
PROJECT_EMAIL = 'vss-apps@eis.utoronto.ca'
MAINTAINER_EMAIL = 'vss-py@eis.utoronto.ca'

PROJECT_GITLAB_GROUP = 'vss'
PROJECT_GITLAB_REPOSITORY = 'py-vss'

PYPI_URL = f'https://pypi.python.org/pypi/{PROJECT_PACKAGE_NAME}'
GITLAB_PATH = f'{PROJECT_GITLAB_GROUP}/{PROJECT_GITLAB_REPOSITORY}'
GITLAB_URL = f'https://gitlab-ee.eis.utoronto.ca/{GITLAB_PATH}'

DOWNLOAD_URL = f'{GITLAB_URL}/archive/{__VERSION__}.zip'
PROJECT_URLS = {
    'Bug Reports': f'{GITLAB_URL}/issues',
    'Documentation': f'{PROJECT_DOCS}/',
    'Source': f'{PROJECT_URL}',
}
STOR_REQUIRE = ['minio==7.1.5']
TESTS_REQUIRE = [
    'flake8==4.0.1',
    'nose==1.3.7',
    'coverage==6.3.2',
    'pytz==2022.1',
    'wheel==0.37.1',  # Otherwise setup.py bdist_wheel does not work
]
TESTS_REQUIRE.extend(STOR_REQUIRE)
DEV_REQUIRE = [
    'flake8==4.0.1',
    'nose==1.3.7',
    'coverage==6.3.2',
    'pytz==2022.1',
    'wheel==0.37.1',
    'sphinx-rtd-theme==1.0.0',
    'Sphinx==4.4.0',
    'bump2version==1.0.1',
]
DEV_REQUIRE.extend(STOR_REQUIRE)

# Allow you to run
# pip install .[test]
# pip install .[dev]
# pip install .[stor]
# to get test dependencies included
EXTRAS_REQUIRE = {
    'test': TESTS_REQUIRE,
    'dev': DEV_REQUIRE,
    'stor': STOR_REQUIRE,
}

MIN_PY_VERSION = '.'.join(map(str, REQUIRED_PYTHON_VER))

setup(
    name=PROJECT_PACKAGE_NAME,
    version=__VERSION__,
    url=PROJECT_URL,
    download_url=DOWNLOAD_URL,
    project_urls=PROJECT_URLS,
    author=PROJECT_AUTHOR,
    author_email=PROJECT_EMAIL,
    maintainer_email=MAINTAINER_EMAIL,
    packages=PACKAGES,
    license=PROJECT_LICENSE,
    python_requires=f'>={MIN_PY_VERSION}',
    install_requires=REQUIRES,
    extras_require=EXTRAS_REQUIRE,
    tests_require=TESTS_REQUIRE,
)



"""Helper module for pyvss.

This helper module contains complimentary functions
to the :py:mod:`pyvss.manager` module
"""
from base64 import b64decode, b64encode
import gzip
from typing import Union

import requests.auth


def normalize_str(string: Union[str, bytes]) -> str:
    """Normalize string."""
    res = string.decode('utf-8') if isinstance(string, bytes) else string
    return res


def is_list_of(lst, o_type=str):
    """Check what kind of list."""
    return (
        bool(lst)
        and isinstance(lst, list)
        and all(isinstance(elem, o_type) for elem in lst)
    )


def compress_encode_string(string_data):
    """Compresses and encodes in base64 a given string.

    :param string_data: string to compress and encode
    :type string_data: str
    :return: compressed and encoded string
    """
    from six import BytesIO

    out = BytesIO()
    with gzip.GzipFile(fileobj=out, mode='w') as f:
        f.write(string_data.encode('utf-8'))
    return b64encode(out.getvalue()).decode()


def decode_uncompress_string(string_gz_encoded):
    """Decompress and decodes a given string.

    :param string_gz_encoded: string to decompress and decode
    :type string_gz_encoded: str
    :return: string
    """
    from six import BytesIO

    string_gz = b64decode(string_gz_encoded)
    file_obj = BytesIO(string_gz)
    return gzip.GzipFile(mode='rb', fileobj=file_obj).read().decode()


def hash_string(string_to_hash):
    """Hashe string using SHA-512.

    :param string_to_hash: string to hash
    :type string_to_hash: str
    :return: hashed string
    """
    import crypt

    if hasattr(crypt, 'mksalt'):
        return crypt.crypt(string_to_hash, crypt.mksalt(crypt.METHOD_SHA512))
    else:
        import string
        import random

        rand = random.SystemRandom()
        salt = ''.join(
            [
                rand.choice(string.ascii_letters + string.digits)
                for _ in range(8)
            ]
        )
        return crypt.crypt(
            string_to_hash,
            "$6$rounds={rounds}${salt}".format(rounds=4096, salt=salt),
        )


class HTTPBasicAuth(requests.auth.HTTPBasicAuth):
    """HTTPBasic AUth class."""

    def __call__(self, r):
        """Override username/password serialization to allow unicode.

        See https://github.com/jakubroztocil/httpie/issues/212
        """
        r.headers['Authorization'] = (
            type(self)
            .make_header(self.username, self.password)
            .encode('latin1')
        )
        return r

    @staticmethod
    def make_header(username, password):
        """Make header for HTTP Auth."""
        credentials = f'{username}:{password}'
        token = b64encode(credentials.encode('utf8')).strip().decode('latin1')
        return 'Basic %s' % token
